/*
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   IBM - initial API and implementation
 */

package org.eclipse.dash.common.build.taskdefs;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;


public class CreateMapFile extends Task
{
  private static final String MAP_FILE_CREATED_SUCCESSFULLY = "Map file created successfully."; //$NON-NLS-1$

  private static final String CVSROOT1TAG = "@cvsRoot1@"; //$NON-NLS-1$

  private static final String CVSTAGTAG = "@cvsTag@"; //$NON-NLS-1$

  private static final String ENTRIESTAG = "@entries@"; //$NON-NLS-1$

  private static final String NL = "\n"; //$NON-NLS-1$

  private static final String COMMA = ","; //$NON-NLS-1$

  private static final String EQ = "="; //$NON-NLS-1$

  private static final String PLUGINA = "plugin@"; //$NON-NLS-1$

  private static final String FRAGMENTA = "fragment@"; //$NON-NLS-1$

  private static final String FRAGMENT = "-fragment$"; //$NON-NLS-1$

  private static final String FEATUREA = "feature@"; //$NON-NLS-1$

  private static final String FEATURE = "-feature$|-feature/"; //$NON-NLS-1$

  private static final String REGEX_DIR = "(\\S+) ?"; //$NON-NLS-1$

  private static final String EMPTY = ""; //$NON-NLS-1$

  private Vector directories;

  private String cvstag;

  private String cvsroot;

  private String templatefile;

  private String targetfile;

  public CreateMapFile()
  {
    directories = new Vector();
    cvstag = EMPTY;
    cvsroot = EMPTY;
    templatefile = EMPTY;
    targetfile = EMPTY;
  }

  public void setDirectories(String d)
  {
    Pattern p = Pattern.compile(REGEX_DIR);
    Matcher m = p.matcher(d);
    while (m.find())
    {
      directories.add(new String(m.group(1)));
      m = p.matcher(m.replaceFirst(EMPTY));
    }
  }

  public void setCvstag(String t)
  {
    cvstag = t;
  }

  public void setCvsroot(String r)
  {
    cvsroot = r;
  }

  public void setTemplatefile(String t)
  {
    templatefile = t;
  }

  public void setTargetfile(String t)
  {
    targetfile = t;
  }

  public void execute() throws BuildException
  {
    String[] entries = new String [directories.size()];
    directories.toArray(entries);

    for (int i = 0; i < entries.length; i++)
    {
      String entry = EMPTY;
      String elementid = basename(entries[i]);

      if (Pattern.compile(FEATURE).matcher(entries[i]).find())
      {
        entry = FEATUREA;
        elementid = Pattern.compile(FEATURE).matcher(elementid).replaceFirst(EMPTY); // s/-feature$//;
      }
      else if (Pattern.compile(FRAGMENT).matcher(entries[i]).find())
      {
        entry = FRAGMENTA;
        entries[i] = Pattern.compile(FRAGMENT).matcher(entries[i]).replaceFirst(EMPTY); // s/-fragment$//;
        elementid = Pattern.compile(FRAGMENT).matcher(elementid).replaceFirst(EMPTY); // s/-fragment$//;
      }
      else
      {
        entry = PLUGINA;
      }

      entries[i] = entry + elementid + EQ + cvstag + COMMA + cvsroot + COMMA + COMMA + entries[i];
    }

    String allentries = join(NL, entries);
    String maptemplate = slurp(templatefile);

    maptemplate = Pattern.compile(ENTRIESTAG, Pattern.MULTILINE).matcher(maptemplate).replaceAll(allentries);
    maptemplate = Pattern.compile(CVSTAGTAG, Pattern.MULTILINE).matcher(maptemplate).replaceAll(cvstag);
    maptemplate = Pattern.compile(CVSROOT1TAG, Pattern.MULTILINE).matcher(maptemplate).replaceAll(cvsroot);

    dump(targetfile, maptemplate);

    System.out.println(MAP_FILE_CREATED_SUCCESSFULLY);
  }

  public String basename(String file)
  {
    File f = new File(file);

    return f.getName();
  }

  private String slurp(String file)
  {
    FileInputStream fin;
    try
    {
      fin = new FileInputStream(file);
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }
    int c;
    String str = EMPTY;

    try
    {
      while ((c = fin.read()) != -1)
      {
        str += (char)c;
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }

    try
    {
      fin.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }

    return str;
  }

  private String join(String glue, String[] arr)
  {
    String str = EMPTY;
    for (int i = 0; i < arr.length; i++)
    {
      str += arr[i] + glue;
    }

    return str;
  }

  private void dump(String file, String str)
  {
    FileOutputStream fout;
    try
    {
      fout = new FileOutputStream(file);
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }
    byte[] b = new byte [str.length()];

    for (int i = 0; i < str.length(); i++)
    {
      b[i] = (byte)str.charAt(i);
    }

    try
    {
      fout.write(b);
    }
    catch (IOException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }

    try
    {
      fout.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      throw new BuildException(e.getMessage());
    }
  }
}
