package org.eclipse.dash.common.build.taskdefs;


import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.DirSet;
import org.apache.tools.ant.types.FileSet;


/**
 * <p>SmartProperty is a handy way to get properties which are defined dynamically, like plugin directories &amp; jar names. 
 * You can also use SmartProperty to massage that value to add/remove text using regular expressions.</p>
 * <p><code>&lt;replace&gt;</code> and <code>&lt;replaceChar&gt;</code> are both supported when using 
 * <code>value=&quot;&quot;</code>, <code>&lt;fileset&gt;</code> and <code>&lt;dirset&gt;</code>,
 * and will be applied in order listed in the task. Only one of <code>value=&quot;&quot;</code>, 
 * <code>&lt;fileset&gt;</code> or <code>&lt;dirset&gt;</code> may be used at a time.</p>
 * <p>In the event that your pattern matches more than one file/dir, your property will be a comma delimited list, 
 * eg., "foo.jar, bar.jar, baz.jar". To prevent this, you can set <code>&lt;smartProperty result="first"/&gt;</code> 
 * or <code>&lt;smartProperty result="last"/gt;</code> to restrict results to a single value.</p>   
 * <p>Examples:</p>
 * <pre>
 * &lt;project basedir="." default="build"&gt;
 *   &lt;dirname property="antfile.dir" file="${ant.file}" /&gt;
 *   &lt;taskdef classname="org.eclipse.emf.build.taskdefs.SmartProperty" classpath="${antfile.dir}/../org.eclipse.emf.build/emf.build_jdk50.jar" name="smartProperty" /&gt;
 *   &lt;target name="build"&gt;
 *     &lt;smartProperty name="foo.dir" result="first"&gt;
 *       &lt;dirset dir="C:\callisto\callisto-plugins\eclipse\plugins" includes="**&#47;org.eclipse.birt.report.*"/&gt;
 *       &lt;replace regex="eclipse" replacement="ECLIPSE"/&gt;
 *       &lt;replace regex="2\.1\.1.*$" replacement="version"/&gt;
 *     &lt;/smartProperty&gt;
 *     &lt;echo message="Got dir: ${foo.dir}"/&gt;
 *
 *     &lt;smartProperty name="foo.file"&gt;
 *       &lt;fileset dir="${antfile.dir}/../org.eclipse.emf.build" includes="**&#47;emf.build_*.jar"/&gt;
 *       &lt;replace regex="jdk\d+" replacement="version"/&gt;
 *       &lt;replace regex="emf" replacement="EMF" replaceFirst="true"/&gt;
 *     &lt;/smartProperty&gt;
 *     &lt;echo message="Got file: ${foo.file}"/&gt;
 *
 *     &lt;smartProperty name="foo.value" value="The quick brown fox jumped over the lazy god."&gt;
 *       &lt;replaceChar oldChar="g" newChar="G"/&gt;
 *       &lt;replace regex="br.*n fox" replacement="jersey cow"/&gt;
 *     &lt;/smartProperty&gt;
 *     &lt;echo message="Got value: ${foo.value}"/&gt;
 *   &lt;/target&gt;
 * &lt;/project&gt;
 * </pre>
 * <pre>
 *    [echo] Got dir: C:\callisto\callisto-plugins\ECLIPSE\plugins\org.ECLIPSE.birt.report.data.oda.jdbc_version
 *    [echo] Got file: C:\eclipse\worspace\org.eclipse.EMF.releng.build\org.eclipse.emf.build\emf.build_version.jar
 *    [echo] Got value: The quick jersey cow jumped over the lazy God.
 * </pre>
 * 
 * @version 2.2.2
 */
public class SmartProperty extends Task
{
  public static class Replace
  {
    String regex;

    String replacement;

    boolean replaceFirst = false;

    public void setRegex(String regex)
    {
      this.regex = regex;
    }

    public void setReplacement(String replacement)
    {
      this.replacement = replacement;
    }

    public void setReplaceFirst(boolean replaceFirst)
    {
      this.replaceFirst = replaceFirst;
    }
  }

  public static class ReplaceChar
  {
    char oldChar;

    char newChar;

    public void setOldChar(char oldChar)
    {
      this.oldChar = oldChar;
    }

    public void setNewChar(char newChar)
    {
      this.newChar = newChar;
    }
  }

  private String name;

  private String value;

  private String result;

  private Vector filesets = new Vector();

  private Vector dirsets = new Vector();

  private List elements = new ArrayList();

  public void setName(String name)
  {
    this.name = name;
  }

  public void setValue(String value)
  {
    this.value = value;
  }

  public void setResult(String result)
  {
    this.result = result;
  }

  public void addFileset(FileSet fileset)
  {
    filesets.add(fileset);
  }

  public void addDirset(DirSet dirset)
  {
    dirsets.add(dirset);
  }

  public void addConfiguredReplace(Replace replace)
  {
    if (replace.replacement == null)
      throw new BuildException("the 'replacement' attribute of the 'replace' element has to be set.");
    if (replace.regex == null)
      throw new BuildException("the 'regex' attribute of the 'replace' element  has to be set.");
    elements.add(replace);
  }

  public void addConfiguredReplaceChar(ReplaceChar replaceChar)
  {
    elements.add(replaceChar);
  }

  protected void validate()
  {
    if (name == null)
    {
      throw new BuildException("The 'name' attribute must be set.");
    }
    if (filesets.size() > 0 && dirsets.size() > 0)
    {
      throw new BuildException("Can only set one of: '<dirset>' or '<fileset>'.");
    }
    if (value == null && filesets.size() < 1 && dirsets.size() < 1)
    {
      throw new BuildException("Either the 'value' attribute or a nested '<dirset>' or '<fileset>' must be set.");
    }
    else if (value != null && (filesets.size() > 0 || dirsets.size() > 0))
    {
      throw new BuildException("Can only set one of: 'value', nested '<dirset>', or nested '<fileset>'.");
    }
  }

  public void execute()
  {
    validate();

    if (dirsets.size() > 0)
    {
      for (Iterator itDSets = dirsets.iterator(); itDSets.hasNext();)
      {
        DirSet dirs = (DirSet)itDSets.next();
        DirectoryScanner ds = dirs.getDirectoryScanner(getProject());
        getMatches(ds, ds.getIncludedDirectories());
      }
    }
    else if (filesets.size() > 0)
    {
      for (Iterator itFSets = filesets.iterator(); itFSets.hasNext();)
      {
        FileSet files = (FileSet)itFSets.next();
        DirectoryScanner ds = files.getDirectoryScanner(getProject());
        getMatches(ds, ds.getIncludedFiles());
      }
    }

    if (value != null)
    {
      for (Iterator i = elements.iterator(); i.hasNext();)
      {
        Object element = i.next();
        if (element instanceof Replace)
        {
          Replace replace = (Replace)element;
          value = replace.replaceFirst ? value.replaceFirst(replace.regex, replace.replacement) : value.replaceAll(
            replace.regex,
            replace.replacement);
        }
        if (element instanceof ReplaceChar)
        {
          ReplaceChar replaceChar = (ReplaceChar)element;
          value = value.replace(replaceChar.oldChar, replaceChar.newChar);
        }
      }
      getProject().setNewProperty(name, value);
    }
    else
    {
      throw new BuildException("No value found for inputs.");
    }
  }

  /* Thanks to 
   * http://ant.apache.org/manual/tutorial-tasks-filesets-properties.html#filesets 
   * for inspiration     
   */
  private void getMatches(DirectoryScanner ds, String[] includedDirs)
  {
    for (int i = 0; i < includedDirs.length; i++)
    {
      String dirname = includedDirs[i].replace('\\', '/');
      dirname = dirname.substring(dirname.lastIndexOf("/") + 1);
      File found = new File(ds.getBasedir(), includedDirs[i]);
      if ("first".equals(result))
      {
        value = found.getAbsolutePath();
        break;
      }
      else if ("last".equals(result))
      {
        value = found.getAbsolutePath();
      }
      else
      {
        value = value == null ? found.getAbsolutePath() : value + ", " + found.getAbsolutePath();
      }
    }
  }
}
