#!/bin/sh
# script to generate Update Manager sites (jars + site.xml), then promote them to download.eclipse
# Copyright \(c\) 2008, IBM. Nick Boldt. codeslave\(at\)ca.ibm.com

#  ! RULES
#    ! for $buildType=R builds, need to only keep latest per component per branch: _r0 (2.2.2, not 2.2.2 + 2.2.1 + 2.2.0)
#    ! for S keep only 2: _r0, _r1
#    ! for I & M, keep 3: _r0, _r1, _r2

#  + setup
#    + delete existing site and any *_r2/ folders
#    + move *_r1 -> *_r2; move *_r0 -> *_r1

#  + generate new content
#    + create new site category fragment using -catgXMLOnly: _rN/category.xml
#    + generate Update zip from *.jar + simplified site.xml (no mirrorsURL, no pack200, no extraneous XML comments)

#  + merge
#    + on local, combine all site content (_r0, _r1, _r2) into one update site:
#    + merge header + category.xml fragments into single site*.xml
#    + store merged site*.xml in CVS

#  + publish
#    + zip composite site*.xml + digest.zip + list of ALL required jars + new jars/pack.gz files & upload to server; unpack on remote
#    + rsync new Update zip into build folder on remote

# requires accompanying properties file, promoteToEclipse.*.properties

norm="\033[0;39m";
grey="\033[1;30m";
green="\033[1;32m";
brown="\033[0;33m";
yellow="\033[1;33m";
blue="\033[1;34m";
cyan="\033[1;36m";
red="\033[1;31m";

##########################################################################################
if [ $# -lt 1 ]; then
	echo " "
	echo "usage: ${0##*/}"
	echo "-d, -projRelengRoot <optional \$CVSROOT for getting propertiesfile; defaults to ";
	echo "                :pserver:anonymous@dev.eclipse.org:/cvsroot/modeling>";
	echo "-c            <optional path to propertiesfile in CVS and version, eg." 
	echo "                org.eclipse.emf/org.eclipse.emf.query.releng/promoteToEclipse.query.properties,R1_1_maintenance"
	echo "-f            <properties file used to run script (default ./promoteToEclipse.properties)>"
	echo "-sub          <REQUIRED: specify subproject as ocl, validation, query, transaction, etc.>"
	echo "-parent       <parent project or 'NONE'; default: modeling>"
	echo "-user         <username on *.eclipse.org (default is $USER)>"
	echo "-q, -Q        <scp, unzip, cvs checkout: -Q (quieter) where possible, -q elsewhere>"
	echo "-OLD          <manipulate features/plugins then generate site.xml; this is the old behaviour from buildUpdate.sh>"
	echo "-debug        <debug this script and ProductUpdateBuilder script [0|1|2] (default 0)>"
	echo "-buildID      <perform UM site jars build on which build ID (eg., I200601191242)>"
	echo "-branch       <branch of the files to be built, eg., 1.0.0, 1.0.1 (overrides value in property file)>"
	echo "-promote      <promote built jars to download> (optional)"
	echo "-notrackstats <do not track stats in site*.xml> (optional)"
	echo "-noclean    <don't delete temp files when done> (optional)"
	echo "-basebuilderBranch  <org.eclipse.releng.basebuilder CVS branch>" #; default to value in build.options.txt if avail>"
	echo "-no4thPart    <build 3-part jars (2.0.1) instead of the default, 4-part jars (2.0.1.I200601191242)>"
	echo " "
	echo "examples: ${0##*/} -sub emf -branch 2.3.2 -buildID R200802051830 -Q \\"
	echo "    -coordsite ganymede -promote \\"
	echo "    2>&1 | tee ~/buildUpdateSite_\`date +%Y%m%d_%H%M%S\`.txt"
	echo " "
	exit 1
fi

# default to default properties file
propertiesFile=""; # value set by -c
propertiesFiles=""; # set of values to iterate 
propertiesFileCmd=""; # contains either "" or "-f /path/to/somefile.properties"
projRelengRoot=":pserver:anonymous@dev.eclipse.org:/cvsroot/modeling"; # default if not specified when building
webuser=$(grep www /etc/passwd | head -1 | perl -pe "s/(^[^:]+):.+$/\1/g");
wwwCVSRep="";
relengCommonScriptsDir=$(cd $(dirname $0); pwd)

# bug 228145: cannot install into Eclipse 3.4 w/ p2 when <feature url="http://..."> used
# TODO: when bug 228145 is fixed, turn this back on to default true
trackstats=false;
quietSSH="-v";
noclean=0;
parent="modeling";
useMaster=0;
quietCVS="-q";
coordsiteSuffix="";
catgXMLOnly="true";
updatesDirSuffix="updates"; # path suffix after /modeling/emf/ or /tools/pdt/; default like /modeling/emf/updates/ but some projects might want /tools/pdt/updates/2.0/ instead

loaded=0;
function loadCVSPropertiesFile()
{
	# Input: org.eclipse.emf/org.eclipse.emf.query.releng/promoteToEclipse.query.properties,R1_1_maintenance
	tmpProperties=`mktemp -d`; tmpProperties=${tmpProperties#/tmp/};
	in=$1;
	if [[ ${in##*,*} ]]; then in=$in,HEAD; fi # default branch is HEAD
	cvsPropertiesPath=$in; cvsPropertiesPath=${cvsPropertiesPath%,*}; # org.eclipse.emf/org.eclipse.emf.query.releng/promoteToEclipse.query.properties
	cvsPropertiesBranch=$in; cvsPropertiesBranch=${cvsPropertiesBranch#*,}; # R1_1_maintenance
	cvsPropertiesFile=${cvsPropertiesPath##*/}; # promoteToEclipse.query.properties
	subprojectName=${cvsPropertiesPath##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; # query
	#echo "cvsPropertiesPath=$cvsPropertiesPath"; echo "cvsPropertiesBranch=$cvsPropertiesBranch"; echo "cvsPropertiesFile=$cvsPropertiesFile"; echo "subprojectName=$subprojectName";
	pushd /tmp >/dev/null;
	cvs -d $projRelengRoot -Q ex -r $cvsPropertiesBranch -d $tmpProperties $cvsPropertiesPath 2>&1 >/dev/null
	if [ -r /tmp/$tmpProperties/$cvsPropertiesFile ]; then 
		subprojectName=${cvsPropertiesFile##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; # set a default value in case not set in the properties file 
		echo "   -c $in [loading $cvsPropertiesFile from $cvsPropertiesBranch, sub = $subprojectName ... "; 
		. /tmp/$tmpProperties/$cvsPropertiesFile;
		propertiesFile=/tmp/$tmpProperties/$cvsPropertiesFile
		propertiesFileCmd="-f $propertiesFile";
		loaded=1;
		echo " done]";
	else 
		echo "[update] Properties file $cvsPropertiesPath not found in CVS for branch/tag $cvsPropertiesBranch. Exiting..."; 
		rm -fr /tmp/$tmp; exit 2; 
	fi; 
	
	if [[ ! $subprojectName ]]; then # if not set in properties file 
		subprojectName=${cvsPropertiesPath##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; # query
	fi
	popd >/dev/null;
}

# Create local variable based on the input
echo " "
echo "[update] Started `date +%H:%M:%S`. Executing with the following options:"
while [ "$#" -gt 0 ]; do
	case $1 in
		'-d'|'-projRelengRoot') projRelengRoot=$2; echo "   $1 $2"; shift 1;;
		'-c')    loadCVSPropertiesFile $2; shift 1;;
		'-f')    propertiesFile=$2;  
			if [[ -r $propertiesFile ]]; then 
				subprojectName=${propertiesFile##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; # set a default value in case not set in the properties file 
				echo "   $1 $2 [loading, sub = $subprojectName ... "; 
				. $propertiesFile; 
				propertiesFileCmd="-f $propertiesFile";
				loaded=1;
				echo "done]";
			else 
				echo "[update] Properties file $propertiesFile not found. Exiting..."; 
				exit 3; 
			fi; 
			shift 1;;
		'-sub')  subprojectName=$2;
			# chain them together in order of priority: subproj specific one, default
			propertiesFiles=$propertiesFile" $relengCommonScriptsDir/promoteToEclipse."$subprojectName".properties ";
			if [[ $loaded -eq 0 ]]; then 
				for propsFile in $propertiesFiles; do 
					if [[ $loaded -eq 0 ]] && [[ -r $propsFile ]]; then 
						propertiesFile=$propsFile 
						echo -n "   $1 $2 [loading $propertiesFile ... ";
						. $propertiesFile; 
						propertiesFileCmd="-f $propertiesFile";
						echo "done]"; 
						loaded=1; 
					fi; 
				done
			fi
			if [[ $loaded -eq 0 ]]; then echo "    [Can't load any of: $propertiesFiles. Must abort!]";exit 4; fi
			shift 1;;

		'-parent')
			echo "   $1 $2"; 
			if [[ $2 == "NONE" ]]; then parent=""; else parent=$2; fi 
			shift 1
			;;

		'-coordsite')
			echo "   $1 $2"; 
			coordsiteSuffix="-"$2; 
			shift 1
			;;
		'-user')
			echo "   $1 $2";
			user=$2;
			shift 1
			;;
		'-buildID')
			echo "   $1 $2";
			buildID=$2;
			shift 1
			;;
		'-branch')
			echo "   $1 $2";
			branch=$2;
			shift 1
			;;
		'-promote')
			echo "   $1";
			promote=1;
			build=1;
			shift 0
			;;
		'-notrackstats')
			echo "   $1";
			trackstats=false;
			shift 0
			;;
		'-nocleanup'|'-noclean')
			echo "   $1";
			noclean=1;
			shift 0
			;;
		'-q') quietCVS=-q; quiet=-q; quietSSH=""; echo "   $1"; shift 0;;
		'-Q') quietCVS=-Q; quiet=-q; quietSSH="-q"; echo "   $1"; shift 0;;
		'-debug')
			echo "   $1 $2";
			debug=$2;
			shift 1
			;;
		'-OLD')
			echo "   $1";
			catgXMLOnly="";
			shift 0
			;;
		'-basebuilderBranch')
			echo "   $1 $2";
			basebuilderBranch=$2;
			shift 1
			;;
		'-no4thPart')
			echo "   $1";
			no4thPart=1;
			shift 0
			;;
		'-wwwCVSRep')
			echo "   $1 $2";
			wwwCVSRep=$2;
			shift 1
			;;
		'-updatesCVSPath')
			echo "   $1 $2";
			updatesCVSPath=$2;
			shift 1
			;;
		'-updatesDirSuffix')
			echo 
			echo "   $1 $2";
			updatesDirSuffix=$2;
			shift 1
			;;
	esac
	shift 1
done

if [[ ! $subprojectName ]] && [[ $propertiesFileCmd ]]; then # if not set in properties file 
	subprojectName=${propertiesFileCmd##*promoteToEclipse.}; subprojectName=${subprojectName%.properties}; 
fi

if [ "$subprojectName" = "" ]; then # no value set!
  echo "[update] No subproject name set in properties file or by -sub flag. Script cannot continue. Exiting...";
  exit 5;
fi

if [ "$branch" = "" ]; then # no value set!
  echo "[update] No branch value set in properties file or by -branch flag. Script cannot continue. Exiting...";
  exit 6;
fi

if [ "$buildID" = "" ]; then # no value set!
  echo "[update] No build ID value set in properties file or by -buildID flag. Script cannot continue. Exiting...";
  exit 7;
fi

##########################################################################################

#need to publish to build.eclipse.org in order to be able to run digest builder
downloadServerFullName=build.eclipse.org

#users (for ssh and cvs connections)
eclipseSSHUser=$user"@"$downloadServerFullName

CVSRep=":ext:"$user"@"$eclipseServerFullName":/cvsroot/modeling"
if [[ ! $wwwCVSRep ]]; then 
	wwwCVSRep=":ext:"$user"@"$eclipseServerFullName":/cvsroot/org.eclipse"
fi

# temp folder
tempfold=$HOME/tmp-buildUpdateSite.sh-$subprojectName-$user-`date +%Y%m%d_%H%M%S`

# working directory for CVS checkouts & building
buildDir=$tempfold/1 ;

webRoot=/var/www/html; if [[ ! -d $webRoot ]]; then webRoot=/opt/public/modeling/public_html/; fi; 
updatesPath=/${localWebDir#$webRoot/}/updates/; # /var/www/html/modeling/$projectName or /var/www/html/tools/$projectName --> /tools/gef/updates/ or /modeling/emf/updates/

##########################################################################################

getBuildIDactual ()
{
	buildIDactual=`find $buildDropsDir/$branch/$buildID -iname "$SDKfilenamepattern"`
	if [[ ! $buildIDactual ]]; then # fall back to *$subprojectName*-SDK-*.zip
		buildIDactual=`find $buildDropsDir/$branch/$buildID -maxdepth 1 -type f -iname "*-$subprojectName*-SDK-*.zip"`
	fi
	buildIDactual=${buildIDactual##*-}; # trim up to -
	buildIDactual=${buildIDactual%%\.zip}; # trim off .zip
 	if [[ $debug -gt 1 ]]; then echo "[update] buildIDactual: $buildIDactual"; fi
}

##########################################################################################

buildIDactual=buildID;
getBuildIDactual;
buildType=${buildID:0:1};

case $buildType in 
	'N'|'M'|'I') buildTypeLabel="interim";;
	'S') buildTypeLabel="milestones";;
	'R') buildTypeLabel="releases";;
esac

# path to update site on build server
localUpdatesWebRoot=$localWebDir/$updatesDirSuffix
localUpdatesWebDir=$localUpdatesWebRoot/$buildTypeLabel
# path to update site on download
updatesRoot=$projectWWWDir/$updatesDirSuffix
updatesDir=$updatesRoot/$buildTypeLabel

if [[ ${SDKfilenamepattern##*incubation*} = "" ]]; then 
	isIncubation="true"; 
else
	isIncubation="false"; 
fi

##########################################################################################

# new hook for using Master Zip to default to -catgXMLOnly
i=0;
while [[ $i -lt ${#filePrefixesToUnzipArray[@]} ]]; do
	prefix=${filePrefixesToUnzipArray[$i]};
	if [[ ${prefix##*Master*} == "" ]]; then
		useMaster=1;
		break;
	fi
	(( i++ ));
done

if [[ $useMaster -eq 1 ]]; then
	catgXMLOnly="true";
fi
 
mkdir -p $buildDir;

if [[ $promote -eq 1 ]]; then
	mkdir -p $buildDir/../2;
fi
cd $buildDir ;

function getBasebuilderBranch ()
{ 
	if [ "x$basebuilderBranch" = "x" ]; then
		if [[ ! $basebuilderBranch ]]; then
			basebuilderBranch=$(cat $buildDropsDir/$branch/$buildID/build.cfg | grep basebuilderBranch); basebuilderBranch=${basebuilderBranch#*basebuilderBranch=}; # echo $basebuilderBranch
			if [[ ! $basebuilderBranch ]]; then
				basebuilderBranch="HEAD";
			fi
		fi
	fi
}

getBasebuilderBranch;

if [[ $debug -gt 0 ]]; then echo "[update] buildDir: $buildDir"; fi # org.eclipse.releng.basebuilder directory
cd $buildDir;

# do we really need the whole basebuilder? Why not just Eclipse platform runtime?
if [[ -d /home/www-data/build/org.eclipse.releng.basebuilder_${basebuilderBranch} ]]; then
	if [[ $debug -gt 0 ]]; then echo "[update] Use /home/www-data/build/org.eclipse.releng.basebuilder_${basebuilderBranch}"; fi
	cp -r /home/www-data/build/org.eclipse.releng.basebuilder_${basebuilderBranch} $buildDir/org.eclipse.releng.basebuilder;
elif [[ -d /home/www-data/build/org.eclipse.releng.basebuilder ]]; then
	if [[ $debug -gt 0 ]]; then echo "[update] Use /home/www-data/build/org.eclipse.releng.basebuilder"; fi
	mkdir -p $buildDir/org.eclipse.releng.basebuilder
	cp -r /home/www-data/build/org.eclipse.releng.basebuilder/* $buildDir/org.eclipse.releng.basebuilder/;
else
	if [[ $debug -gt 0 ]]; then echo "[update] Get org.eclipse.releng.basebuilder from dev using $basebuilderBranch"; fi
	cd $buildDir; cvs -d :pserver:anonymous@dev.eclipse.org:/cvsroot/eclipse -Q ex -r $basebuilderBranch org.eclipse.releng.basebuilder 2>&1 >/dev/null;
fi
basebuilderDir=$buildDir/org.eclipse.releng.basebuilder
if [[ ! -d $basebuilderDir ]]; then
	echo "[update] \$basebuilderDir does not exist in $basebuilderDir: must abort."; exit 1;
fi 
if [[ $debug -gt 0 ]]; then echo "[update] basebuilderDir: $basebuilderDir"; fi # org.eclipse.releng.basebuilder directory
cd $buildDir;

relengGeneratorsDir=/home/www-data/build/org.eclipse.releng.generators
if [[ ! -d $relengGeneratorsDir ]]; then relengGeneratorsDir=/opt/public/modeling/org.eclipse.releng.generators; fi
if [[ -d $relengGeneratorsDir ]]; then 
	if [[ $debug -gt 0 ]]; then echo "[update] Update $relengGeneratorsDir"; fi
	cd $relengGeneratorsDir; cvs -Q up -Pd 2>&1 >/dev/null;
	mkdir -p $buildDir/org.eclipse.releng.generators/;
	rsync -arqq $relengGeneratorsDir/ $buildDir/org.eclipse.releng.generators/;
else
	if [[ $debug -gt 0 ]]; then echo "[update] Get $relengGeneratorsCVSPath"; fi
	cd $buildDir; cvs -d $CVSRep -Q ex -d org.eclipse.releng.generators -r HEAD $relengGeneratorsCVSPath 2>&1 >/dev/null; # on local
fi
relengGeneratorsDir=$buildDir/org.eclipse.releng.generators
if [[ ! -d $relengGeneratorsDir ]]; then
	echo "[update] \$relengGeneratorsDir does not exist in $relengGeneratorsDir: must abort."; exit 1;
fi 
if [[ $debug -gt 0 ]]; then echo "[update] relengGeneratorsDir: $relengGeneratorsDir"; fi # org.eclipse.releng.basebuilder directory
cd $buildDir;

# need override here for GMF paths?
# one update site per project
if [[ ! $updatesCVSPath ]]; then
	if [[ $parent ]] && [[ $parent != "NONE" ]]; then 
		updatesCVSPath=www/$parent/$projectName/$updatesDirSuffix
	else
		updatesCVSPath=www/$projectName/$updatesDirSuffix
	fi
fi
echo "[update] [`date +%H:%M:%S`] Check out $updatesCVSPath/* from $wwwCVSRep"
cd $buildDir/../1;
cvs -d $wwwCVSRep -Q co -P -d site $updatesCVSPath 2>&1 >/dev/null;
	
function createHeader ()
{
	dd=$1; # dir
	ff=$2; # file
	pushd $dd >/dev/null;
	if [[ $debug -gt 0 ]]; then echo -e "[update] Create $blue$ff$norm ($grey$updatesPath$norm, $grey$buildTypeLabel$norm) ..."; fi
	echo -n '' > $ff
	echo '<?xml version="1.0" encoding="UTF-8"?>' >> $ff
	if [[ $promote -eq 1 ]]; then 
		echo '<site pack200="true" digestURL="http://download.eclipse.org'$updatesPath$buildTypeLabel'/" mirrorsURL="http://www.eclipse.org/downloads/download.php?file='$updatesPath$buildTypeLabel'/site.xml&amp;format=xml">' >> $ff
	else # [236762] p2 won't fallback to build server if no mirrors found
		echo '<site pack200="true">' >> $ff
	fi 
	echo '	<!-- to track downloads, use absolute paths instead: http://www.eclipse.org/downloads/download.php?r=1&amp;file='$updatesPath' ... -->' >> $ff
	echo '' >> $ff
	echo '	<description url="http://download.eclipse.org'$updatesPath$buildTypeLabel'/site.xml">' >> $ff
	echo '	To install these plugins, point Eclipse at this site. ' >> $ff
	echo '	</description>' >> $ff
	echo '' >> $ff
	popd >/dev/null;
}

sitexmlPre=site
if [[ ! -f site/header.${buildTypeLabel}.$sitexmlPre.xml ]]; then
	createHeader $buildDir/../1/site header.${buildTypeLabel}.$sitexmlPre.xml 
fi
if [[ ! -f site/$sitexmlPre.xml ]]; then
	if [[ $debug -gt 0 ]]; then echo "[update] Create new $sitexmlPre.xml"; fi
	pushd site >/dev/null; cp header.${buildTypeLabel}.$sitexmlPre.xml $sitexmlPre.xml; echo "</site>" >> $sitexmlPre.xml; cvs -Q add $sitexmlPre.xml 2>&1 >/dev/null; popd >/dev/null;
fi
if [[ $coordsiteSuffix ]]; then
	if [[ ! -f site/$sitexmlPre${coordsiteSuffix}.xml ]]; then
		if [[ $debug -gt 0 ]]; then echo "[update] Create new $sitexmlPre${coordsiteSuffix}.xml"; fi
		pushd site >/dev/null; cp $sitexmlPre.xml $sitexmlPre${coordsiteSuffix}.xml; cvs -Q add $sitexmlPre${coordsiteSuffix}.xml 2>&1 >/dev/null; popd >/dev/null;
	fi
fi

cd $buildDir;
#run ant script

if [[ $buildIDactual != $buildID ]]; then
	buildDesc=$buildIDactual; # eg., 2.0.1RC1 != M200409081700
else
	buildDesc=$branch; # eg., 2.0.1
fi

# for use with plugins as jars: unpack SDK zips and then replace foo.jar with foo/ folders instead
rm -fr ${relengGeneratorsDir}/updateJars
mkdir -p ${relengGeneratorsDir}/updateJars/site

if [[ $debug -gt 0 ]]; then echo -e "[update] (1) Unpacking zip(s) to collect features & plugins ($blue\$filePrefixesToUnzipArray$norm) ..."; fi
index=0;
element_count=${#filePrefixesToUnzipArray[@]}
zipsUnpacked=0;
zipfilenames="";
while [ "$index"  -lt "$element_count" ]; do
	zipfilename=$(cd $buildDropsDir/$branch/$buildID/; find . -maxdepth 1 -type f -iname ${filePrefixesToUnzipArray[$index]}"$buildIDactual.zip");
	zipfilename=${zipfilename#./};
	if [[ ! -f $buildDropsDir/$branch/$buildID/$zipfilename ]] && [[ ! -f $buildDropsDir/$branch/$buildID/${zipfilename/-Master-/-Update-} ]]; then
		if [[ $debug -gt 0 ]]; then
			if [[ $zipfilename ]]; then 
				echo -e "[update] ${brown}*!*${norm} No such zip: $buildDropsDir/$branch/$buildID/$red$zipfilename$norm or $red"${zipfilename/-Master-/-Update-}"$norm ${brown}*!*${norm}";
			else
				echo -e "[update] ${brown}*!*${norm} No such zip: $buildDropsDir/$branch/$buildID/$red"${filePrefixesToUnzipArray[$index]}"$buildIDactual.zip$norm ${brown}*!*${norm}";
			fi		
		fi
	else
		if [[ ! -f $buildDropsDir/$branch/$buildID/$zipfilename ]]; then
			zipfilename=${zipfilename/\-Master\-/\-Update\-};
			echo -e "[update] [`date +%H:%M:%S`] Unpack $blue$zipfilename$norm ..."
			mkdir -p ${relengGeneratorsDir}/updateJars 
			unzip -uo -qq $buildDropsDir/$branch/$buildID/$zipfilename -d ${relengGeneratorsDir}/updateJars/eclipse
		else
			echo -e "[update] [`date +%H:%M:%S`] Unpack $blue$zipfilename$norm ..."
			unzip -uo -qq $buildDropsDir/$branch/$buildID/$zipfilename -d ${relengGeneratorsDir}/updateJars
		fi
		zipfilenames=$zipfilenames" "$zipfilename;
		(( zipsUnpacked += 1 ));
	fi
	(( index += 1 ));
done
if [[ $zipsUnpacked -eq 0 ]]; then # fall back to SDKfilenamepattern to find correct zips
	if [[ $debug -gt 0 ]]; then echo -e "[update] (2) Unpacking zip(s) to collect features & plugins ($blue\$SDKfilenamepattern$norm) ..."; fi
	for zipfilename in $(cd $buildDropsDir/$branch/$buildID/; find . -maxdepth 1 -type f -iname "$SDKfilenamepattern"); do
		zipfilename=${zipfilename#./};
		if [[ ! -f $buildDropsDir/$branch/$buildID/$zipfilename ]]; then
			if [[ $debug -gt 0 ]]; then echo -e "[update] ${brown}*!*${norm} No such zip: $buildDropsDir/$branch/$buildID/$zipfilename ${brown}*!*${norm}"; fi
		else
			echo -e "[update] [`date +%H:%M:%S`] Unpack $yellow$zipfilename$norm ..."
			unzip -uo -qq $buildDropsDir/$branch/$buildID/$zipfilename -d ${relengGeneratorsDir}/updateJars
			zipfilenames=$zipfilenames" "$zipfilename;
			(( zipsUnpacked += 1 ));
		fi
	done
fi		
if [[ $zipsUnpacked -eq 0 ]]; then # fall back to *$subprojectName*-SDK-*.zip
	if [[ $debug -gt 0 ]]; then echo -e "[update] (3) Unpacking zip(s) to collect features & plugins (${blue}\"*-$subprojectName*-SDK-*.zip\"$norm) ..."; fi
	for zipfilename in $(cd $buildDropsDir/$branch/$buildID/; find . -maxdepth 1 -type f -iname "*-$subprojectName*-SDK-*.zip"); do
		zipfilename=${zipfilename#./};
		if [[ ! -f $buildDropsDir/$branch/$buildID/$zipfilename ]]; then
			if [[ $debug -gt 0 ]]; then echo -e "[update] ${brown}*!*${norm} No such zip: $buildDropsDir/$branch/$buildID/$zipfilename ${brown}*!*${norm}"; fi
		else
			echo -e "[update] [`date +%H:%M:%S`] Unpack $red$zipfilename$norm ..."
			unzip -uo -qq $buildDropsDir/$branch/$buildID/$zipfilename -d ${relengGeneratorsDir}/updateJars
			zipfilenames=$zipfilenames" "$zipfilename;
			(( zipsUnpacked += 1 ));
		fi
	done
fi
if [[ $zipsUnpacked -eq 0 ]]; then
	echo -e "[update] ${red}*!* No zips found to unpack.${norm} Check ${yellow}filePrefixesToUnzipArray${norm} in ${yellow}promoteToEclipse.$subprojectName.properties${norm}. Must abort! ${red}*!*${norm}"; exit 1;
fi

# skip if -catgXMLOnly ?
if [[ ! $catgXMLOnly ]]; then 
	pluginjars=`find ${relengGeneratorsDir}/updateJars/eclipse/plugins -maxdepth 1 -name "*.jar"`
	for jarfile in $pluginjars; do
		folder=${jarfile%\.jar}
		echo "Unpacking "${jarfile#*plugins/}" ...";
		#echo "Unpacking $jarfile into $folder ...";
		unzip -uo -qq $jarfile -d $folder;
		#echo "Removing $jarfile ...";
		rm -fr $jarfile;
	done
fi

# TODO eventually, this could be simplified down to just an XML generator - no Java required!
if [[ $debug -gt 0 ]]; then 
	echo -e "[update] [`date +%H:%M:%S`] Launch productUpdateBuilder.jar to generate site XML for $blue$buildID$norm...";
else
	echo -e "[update] [`date +%H:%M:%S`] Generate site XML for $blue$buildID$norm ${grey}(use -debug 1 for details)${norm}.";
fi
# java home & vm used to run the build.  Defaults to java on system path
## use Sun 1.4 or 5.0 here; 5.0 used to throw NPE (SAXParser problem); IBM 1.4 crashed old server
javaHome=/opt/sun-java2-1.4
vm=$javaHome/bin/java
bootclasspath="$javaHome/jre/lib/*.jar:${relengGeneratorsDir}/buildTools.jar:${relengGeneratorsDir}/productUpdateBuilder.jar"

# different ways to get the launcher and Main class
if [[ -f $basebuilderDir/startup.jar ]]; then 
  cpAndMain="$basebuilderDir/startup.jar org.eclipse.core.launcher.Main"; # up to M4_33
elif [[ -f $basebuilderDir/plugins/org.eclipse.equinox.launcher.jar ]]; then
  cpAndMain="$basebuilderDir/plugins/org.eclipse.equinox.launcher.jar org.eclipse.equinox.launcher.Main"; # M5_33
else
  cpAndMain=`find $basebuilderDir/ -name "org.eclipse.equinox.launcher_*.jar" | sort | head -1`" org.eclipse.equinox.launcher.Main"; 
fi

command="$vm -cp $cpAndMain"
command=$command" -application org.eclipse.ant.core.antRunner"
command=$command" -f $buildDir/$antScript $target"
command=$command" -Dbootclasspath=$bootclasspath"
command=$command" -Dtimestamp=$buildID"
command=$command" -DbuildDesc=$buildDesc"
command=$command" -DprojectName=$projectName"
command=$command" -DsubprojectName=$subprojectName"
command=$command" -DupdatesPath=$updatesPath"
command=$command" -Dincubation=$isIncubation"
command=$command" -Ddebug=$debug"
command=$command" -Dtrackstats=$trackstats"
command=$command" -DpropertiesFile=$propertiesFile"
#command=$command" -Dbase.builder=$basebuilderDir"

# if $catgXMLOnly, $sitexml value should be category.xml instead
if [[ $catgXMLOnly ]]; then
	genxml="category";
else
	genxml="site";
fi

sitexml=$genxml".xml"; 
if [[ $buildType == "R" ]] || [[ $no4thPart -eq 1 ]] || [[ $buildIDactual == $branch ]]; then 
	command=$command" -DfourthPart=no4thPart"
	command=$command" -Dsitexml=$sitexml"
	no4thPart=1;
else
	command=$command" -DfourthPart=add4thPart"
	command=$command" -Dsitexml=$sitexml"
fi

if [[ $catgXMLOnly ]]; then command=$command" -DcatgXMLOnly=$catgXMLOnly"; fi

if [[ -f $buildDir/site/$sitexml ]]; then
	cp -f $buildDir/site/$sitexml ${relengGeneratorsDir}/updateJars/site/
fi

if [[ $debug -gt 0 ]]; then 
	echo "$command" | perl -pe "s/ -/\n  -/g" ; # pretty printing
	$command;
else
	$command 2>&1 >/dev/null;
fi

# if we've generated category.xml, we need a final site-interim.xml / site-interim-ganymede.xml too
sitexmlOut=site.xml

# create both a coordsite (no stat tracking URLs in site.xml) and a non-coordsite (with download stat tracking)
if [[ $coordsiteSuffix ]]; then
	coordsitexml=${sitexml%\.xml}${coordsiteSuffix}".xml"; # eg., site-interim-europa.xml
	if [[ $trackstats == "true" ]]; then
		cat $buildDir/site/$sitexml | perl -pe "s#http://www.eclipse.org/downloads/download.php\?r=1\&amp;file=${updatesPath}features/#features/#g" > $buildDir/site/$coordsitexml
		#diff $buildDir/site/$sitexml $buildDir/site/$coordsitexml;
	else
		cp $buildDir/site/$sitexml $buildDir/site/$coordsitexml;
	fi
	coordsitexmlOut=site${coordsiteSuffix}.xml
fi

# if we didn't generate the jars/features, we have to manually copy them over
if [[ $catgXMLOnly ]]; then 
	num=$(ls ${relengGeneratorsDir}/updateJars/eclipse/plugins/*.jar 2>&1 | grep -v "No such file or directory" | grep -c ".");
	if [[ $num -gt 0 ]]; then
		if [[ $debug -gt 0 ]]; then echo -e "[update] [`date +%H:%M:%S`] Copy ${blue}$num${norm} jarred plugins ..."; fi
		mkdir -p ${relengGeneratorsDir}/updateJars/site/plugins/
		cp ${relengGeneratorsDir}/updateJars/eclipse/plugins/*.jar ${relengGeneratorsDir}/updateJars/site/plugins/
		num=$(ls ${relengGeneratorsDir}/updateJars/eclipse/plugins/*.jar.pack.gz 2>&1 | grep -v "No such file or directory" | grep -c ".");
		if [[ $num -gt 0 ]]; then
			if [[ $debug -gt 0 ]]; then echo -e "[update] [`date +%H:%M:%S`] Copy ${blue}$num${norm} packed plugins ..."; fi
			cp ${relengGeneratorsDir}/updateJars/eclipse/plugins/*.jar.pack.gz ${relengGeneratorsDir}/updateJars/site/plugins/
		fi
	fi
	num=$(find ${relengGeneratorsDir}/updateJars/eclipse/plugins/ -mindepth 1 -maxdepth 1 -type d -and -not -name "." | grep -c "plugins");
	if [[ $num -gt 0 ]]; then
		featdirs=$(find ${relengGeneratorsDir}/updateJars/eclipse/plugins/ -mindepth 1 -maxdepth 1 -type d -and -not -name ".");
		if [[ $debug -gt 0 ]]; then echo -e "[update] [`date +%H:%M:%S`] Copy and jar ${blue}$num${norm} plugins ..."; fi
		mkdir -p ${relengGeneratorsDir}/updateJars/site/plugins/
		for featdir in $featdirs; do
		  #echo -n "        Zip $featdir as ${featdir}.jar ...";
		  pushd $featdir >/dev/null; 
		  zip -9rq ${featdir}.jar *;
		  mv ${featdir}.jar ${relengGeneratorsDir}/updateJars/site/plugins/ 
		  popd >/dev/null;
		  #echo " done.";
		done
	fi

	if [[ $useMaster -eq 1 ]] && [[ $(echo $zipfilenames | egrep "\-Update\-|\-Master\-") ]]; then # from Master zip, we already have feature jars
		num=$(ls ${relengGeneratorsDir}/updateJars/eclipse/features/*.jar 2>&1 | grep -v "No such file or directory" | grep -c ".");
		if [[ $debug -gt 0 ]]; then echo -e "[update] [`date +%H:%M:%S`] Copy ${blue}$num${norm} jarred features ..."; fi
		mkdir -p ${relengGeneratorsDir}/updateJars/site/features/
		cp ${relengGeneratorsDir}/updateJars/eclipse/features/*.jar ${relengGeneratorsDir}/updateJars/site/features/
		num=$(ls ${relengGeneratorsDir}/updateJars/eclipse/features/*.jar.pack.gz 2>&1 | grep -v "No such file or directory" | grep -c ".");
		if [[ $num -gt 0 ]]; then
			if [[ $debug -gt 0 ]]; then echo -e "[update] [`date +%H:%M:%S`] Copy ${blue}$num${norm} packed features ..."; fi
			cp ${relengGeneratorsDir}/updateJars/eclipse/features/*.jar.pack.gz ${relengGeneratorsDir}/updateJars/site/features/
		fi
	else
		num=$(find ${relengGeneratorsDir}/updateJars/eclipse/features/ -mindepth 1 -maxdepth 1 -type d -and -not -name "." | grep -c "features");
		featdirs=$(find ${relengGeneratorsDir}/updateJars/eclipse/features/ -mindepth 1 -maxdepth 1 -type d -and -not -name ".");
		if [[ $debug -gt 0 ]]; then echo -e "[update] [`date +%H:%M:%S`] Copy and jar ${blue}$num${norm} features ..."; fi
		mkdir -p ${relengGeneratorsDir}/updateJars/site/features/
		for featdir in $featdirs; do
		  #echo -n "        Zip $featdir as ${featdir}.jar ...";
		  pushd $featdir >/dev/null; 
		  zip -9rq ${featdir}.jar *;
		  mv ${featdir}.jar ${relengGeneratorsDir}/updateJars/site/features/ 
		  popd >/dev/null;
		  #echo " done.";
		done
	fi
fi

function cleanupAndExit ()
{
	echo "";
	if [[ $noclean -eq 0 ]]; then
		# cleanup temp space
		if [[ -d $tempfold/ ]]; then
			rm -fr $tempfold/;
			if [[ $debug -gt 0 ]]; then echo "[update] Temporary files purged from $tempfold/."; fi
		fi
		if [[ $tmpProperties ]] && [[ -d /tmp/$tmpProperties ]]; then
			rm -fr /tmp/$tmpProperties
			if [[ $debug -gt 0 ]]; then echo "[update] Temporary files purged from /tmp/$tmpProperties/."; fi
		fi
		if [[ -f $localUpdatesWebRoot/${siteLabel}-${buildID}.zip ]]; then 
			rm -f $localUpdatesWebRoot/${siteLabel}-${buildID}.zip
			if [[ $debug -gt 0 ]]; then echo "[update] Temporary zip $localUpdatesWebRoot/${siteLabel}-${buildID}.zip purged."; fi
		fi
		if [[ $localUpdatesWebDir ]] && [[ -d $localUpdatesWebDir/${siteLabel}_rN ]]; then
			rm -fr $localUpdatesWebDir/${siteLabel}_rN;
			if [[ $debug -gt 0 ]]; then echo "[update] Temporary files purged from $localUpdatesWebDir/${siteLabel}_rN/."; fi
		fi
	else
		if [[ $tmpProperties ]]; then
			echo "[update] Temporary files left in $tempfold/ and /tmp/$tmpProperties/."
		else
			echo "[update] Temporary files left in $tempfold/."
		fi
		if [[ -f $localUpdatesWebRoot/${siteLabel}-${buildID}.zip ]]; then 
			echo "[update] Temporary zip $localUpdatesWebRoot/${siteLabel}-${buildID}.zip not purged.";
		fi
		if [[ -f $localUpdatesWebRoot/${siteLabel}-${buildID}.zip ]]; then 
			echo "[update] Temporary zip $localUpdatesWebRoot/${siteLabel}-${buildID}.zip not purged.";
		fi
		if [[ $localUpdatesWebDir ]] && [[ -d $localUpdatesWebDir/${siteLabel}_rN ]]; then 
			echo "[update] Temporary folder $localUpdatesWebRoot/${siteLabel}_rN/ not purged.";
		fi
	fi

	echo "[update] ${0##*/} completed on: `date +%Y%m%d\ %H\:%M\:%S`"
	exit;
}

############################################################################################################

branchNum=${branch%.*}; # eg., for 2.1.3, want 2.1 
siteLabel=$projectName-$subprojectName-$branchNum-${buildType}; # eg., emf-query-2.1-M

echo "[update] [`date +%H:%M:%S`] Copy new jars & site/* to $localUpdatesWebDir/${siteLabel}_rN ..."
mkdir -p $localUpdatesWebDir/${siteLabel}_rN;
pushd ${relengGeneratorsDir}/updateJars/site >/dev/null; cp -r . $localUpdatesWebDir/${siteLabel}_rN; popd >/dev/null; # jars
pushd $buildDir/site >/dev/null; 
for c in $(cd $buildDir/site; find . -name "category*.xml"); do
	cp -r $c $localUpdatesWebDir/${siteLabel}_rN/${c/-$buildTypeLabel/}; # from category-interim.xml -> category.xml 
done
popd >/dev/null; # category*.xml

# prevent duplication -- instead of just blinding rolling the backups, 
# compare latest (_rN) w/ older ones (_r0 - _r2) to determine if new content 
# is really new or a dupe
newSiteContent=true
if [[ ! -d $localUpdatesWebDir/${siteLabel}_r0 ]]; then
	newSiteNumber=0;
else
	newSiteNumber=0;
	for n in 0 1 2; do
		if [[ -d $localUpdatesWebDir/${siteLabel}_r${n} ]]; then
			diff=$(diff -qr $localUpdatesWebDir/${siteLabel}_rN $localUpdatesWebDir/${siteLabel}_r${n})
			if [[ ! $diff ]]; then
				newSiteNumber=$n; newSiteContent=false; break;
			fi
		fi
	done
fi

if [[ $newSiteContent == "true" ]]; then
	if [[ -d $localUpdatesWebDir/${siteLabel}_r2 ]] || [[ -d $localUpdatesWebDir/${siteLabel}_r1 ]] || [[ -d $localUpdatesWebDir/${siteLabel}_r0 ]]; then
		echo "[update] [`date +%H:%M:%S`] Cycle backups in $localUpdatesWebDir"
		if [[ -d $localUpdatesWebDir/${siteLabel}_r2 ]]; then
			if [[ $debug -gt 0 ]]; then echo "         Delete ${siteLabel}_r2 ..."; fi 
			rm -fr $localUpdatesWebDir/${siteLabel}_r2; 
		fi
		if [[ -d $localUpdatesWebDir/${siteLabel}_r1 ]]; then
			if [[ $debug -gt 0 ]]; then echo "         Move ${siteLabel}_r1 to _r2 ..."; fi 
			mv -f $localUpdatesWebDir/${siteLabel}_r1 $localUpdatesWebDir/${siteLabel}_r2; 
		fi
		if [[ -d $localUpdatesWebDir/${siteLabel}_r0 ]]; then
			if [[ $debug -gt 0 ]]; then echo "         Move ${siteLabel}_r0 to _r1 ..."; fi 
			mv -f $localUpdatesWebDir/${siteLabel}_r0 $localUpdatesWebDir/${siteLabel}_r1; 
		fi
	fi
	if [[ $newSiteNumber -ge 0 ]]; then
		if [[ $debug -gt 0 ]]; then echo "[update] [`date +%H:%M:%S`] Copy $localUpdatesWebDir/${siteLabel}_rN to ${siteLabel}_r${newSiteNumber} ..."; fi
		cp -r $localUpdatesWebDir/${siteLabel}_rN $localUpdatesWebDir/${siteLabel}_r${newSiteNumber}
	fi
	
else
	echo -e "[update] ${yellow}Generated site content is not new. Replace content in ${blue}${siteLabel}_r${newSiteNumber} ...${norm}";
	if [[ $newSiteNumber -ge 0 ]]; then
		if [[ $debug -gt 0 ]]; then echo "[update] [`date +%H:%M:%S`] Move $localUpdatesWebDir/${siteLabel}_rN to ${siteLabel}_r${newSiteNumber} ..."; fi
		rm -fr $localUpdatesWebDir/${siteLabel}_r${newSiteNumber};
		cp -r $localUpdatesWebDir/${siteLabel}_rN $localUpdatesWebDir/${siteLabel}_r${newSiteNumber}
	fi
fi


if [[ $debug -gt 0 ]]; then echo ""; fi
echo "[compose] [`date +%H:%M:%S`] Delete existing features & plugins in $localUpdatesWebDir ..."
rm -fr $localUpdatesWebDir/features $localUpdatesWebDir/plugins;
mkdir -p $localUpdatesWebDir/features $localUpdatesWebDir/plugins;

componentSiteDirs="";
case $buildType in 
	'N'|'M'|'I') for n in 0 1 2; do componentSiteDirs=$componentSiteDirs" "$(find $localUpdatesWebDir -maxdepth 1 -type d -name "${projectName}-*-?_r${n}" | sort -r); done;;
	'S') 		 for n in 0 1;   do componentSiteDirs=$componentSiteDirs" "$(find $localUpdatesWebDir -maxdepth 1 -type d -name "${projectName}-*-${buildType}_r${n}" | sort -r); done;;
	'R')		 for n in 0;     do componentSiteDirs=$componentSiteDirs" "$(find $localUpdatesWebDir -maxdepth 1 -type d -name "${projectName}-*-${buildType}_r${n}" | sort -r); done;;
esac
for d in $componentSiteDirs; do
	(( numc += 1 ));
done

echo -e "[compose] [`date +%H:%M:%S`] Merge $blue$numc$norm components' jars into $localUpdatesWebDir/"
cnt=0;
for d in $componentSiteDirs; do
	(( cnt += 1 ));
	if [[ -d $d/features ]]; then
		warn="   "; if [[ ! $(find $d/features -type f -name "*.jar*") ]]; then warn="${red}*!*$norm"; fi  
		if [[ $debug -gt 0 ]]; then echo -n -e "$warn [$cnt/$numc] From $brown${d#$localUpdatesWebDir/}$norm/features/ ..."; fi
		cp -r $d/features/* $localUpdatesWebDir/features;
	fi
	if [[ -d $d/plugins ]]; then
		warn=" "; if [[ ! $(find $d/plugins -type f -name "*.jar*") ]]; then warn=" ${red}*!*$norm "; fi  
	if [[ $debug -gt 0 ]]; then echo -n -e "${warn}plugins/ ..."; fi
		cp -r $d/plugins/* $localUpdatesWebDir/plugins;
	fi
	if [[ -d $d/features ]] || [[ -d $d/plugins ]]; then
		if [[ $debug -gt 0 ]]; then echo ""; fi
	fi
done

#################################################################################################

# do not create an Update zip these unless there's a Master/Update zip already; must be doing jar signing + packing as a prereq
masterZip=$(echo $zipfilenames | egrep "\-Update\-|\-Master\-" | tail -1); # zipfilenames contains real zip file names (no path) 
if [[ $masterZip ]]; then #add category*.xml and site*.xml into Master zip, and rename it
	command="$relengCommonScriptsDir/buildUpdateSiteXML.sh -label $buildTypeLabel -header $buildDir/site/header.$buildTypeLabel"
	command=$command" -targetDir $localUpdatesWebDir/${siteLabel}_rN $localUpdatesWebDir/${siteLabel}_rN"; # generate single
	if [[ $coordsiteSuffix ]]; then
		command=$command" -coordsite "${coordsiteSuffix:1};
	fi
	if [[ $debug -gt 0 ]]; then 
		echo -e "[compose] [`date +%H:%M:%S`] Run buildUpdateSiteXML.sh to create site*.xml ..." 
		command=$command" -debug $debug";
		if [[ $debug -gt 1 ]]; then 
			echo "$command" | perl -pe "s/ -/\n  -/g" ; # pretty printing
		fi
	fi
	$command;

	if [[ $debug -gt 0 ]]; then 
		echo ""; 
		echo "[fix zip] [`date +%H:%M:%S`] Generate Update zip ... ";
	else 
		echo -n "[fix zip] [`date +%H:%M:%S`] Generate Update zip ... "; 
	fi
	updateZip=${masterZip/-Master-/-Update-}
	pushd $localUpdatesWebDir/${siteLabel}_rN >/dev/null;
	if [[ $debug -gt 0 ]]; then echo "          Remove download tracking URLs in site.xml (bug 228145) ..."; fi 
	# bug 228145: if site.xml contains http://www.eclipse.org/downloads/... instead of just features/, can't install
	# also, remove digestURL, mirrorsURL, pack200 & comments (because we're dealing with a LOCAL update site archive) 
	mv -f site.xml site-full.xml;
	if [[ $trackstats == "true" ]]; then
		cat site-full.xml | \
			perl -pe "s#http://www.eclipse.org/downloads/download.php\?r=1\&amp;file=${updatesPath}features/#features/#g" | \
			perl -pe 's# digestURL="[^"]+"##g' | \
			perl -pe 's# mirrorsURL="[^"]+"##g' | \
			perl -pe 's# pack200="[^"]+"##g' | \
			perl -pe 's#<!-- [^-]+ -->##' \
			> site.xml;
	else
		cat site-full.xml | \
			perl -pe 's# digestURL="[^"]+"##g' | \
			perl -pe 's# mirrorsURL="[^"]+"##g' | \
			perl -pe 's# pack200="[^"]+"##g' | \
			perl -pe 's#<!-- [^-]+ -->##' \
			> site.xml;
	fi
	if [[ $debug -gt 0 ]]; then echo "          Add jars + site*.xml into $updateZip ..."; fi 
	sudo -u $webuser zip -9rq $buildDropsDir/$branch/$buildID/$updateZip features/*.jar plugins/*.jar site.xml;
	mv -f site-full.xml site.xml;
	popd >/dev/null; 
	if [[ $debug -gt 0 ]]; then echo "          Recalculate md5sum ..."; fi
	pushd $buildDropsDir/$branch/$buildID >/dev/null;
	md5tmp=`mktemp`;
	md5sum ${updateZip} > $md5tmp; chmod 644 $md5tmp; 
	sudo -u $webuser cp -f $md5tmp $buildDropsDir/$branch/$buildID/${updateZip}.md5;
	rm -f $md5tmp;
	popd >/dev/null;
	if [[ $debug -gt 0 ]]; then 
		echo -e "[fix zip] [`date +%H:%M:%S`] Update zip is: $blue$branch/$buildID$norm/$blue${updateZip}$norm";
	else
		echo -e "$blue$branch/$buildID$norm/$blue${updateZip}$norm";
	fi
fi

command="$relengCommonScriptsDir/buildUpdateSiteXML.sh -label $buildTypeLabel -header $buildDir/site/header.$buildTypeLabel"
command=$command" -targetDir $localUpdatesWebDir $componentSiteDirs"; # generate composite
if [[ $coordsiteSuffix ]]; then
	command=$command" -coordsite "${coordsiteSuffix:1};
fi
if [[ $debug -gt 0 ]]; then 
	echo -e "[compose] [`date +%H:%M:%S`] Run buildUpdateSiteXML.sh to create site*.xml ..." 
	command=$command" -debug $debug";
	if [[ $debug -gt 1 ]]; then 
		echo "$command" | perl -pe "s/ -/\n  -/g" ; # pretty printing
	fi
fi
$command;
chgrp -f -R $buildUserGroup $localUpdatesWebDir/site*.xml;
chmod -f -R g+w $localUpdatesWebDir/site*.xml;
	 
#################################################################################################

# generate list of all the features/ and plugins/ files so that we can compare this list 
# with what's on REMOTE site, in order to know what can safely be deleted from that site.
pushd $localUpdatesWebDir >/dev/null; 
find features/ plugins/ -type f > $localUpdatesWebDir/jarlist.clean.`hostname -f`.txt; 
popd >/dev/null; 

#################################################################################################

# generate local site digest 
rm -f $localUpdatesWebDir/digest.zip
command="$relengCommonScriptsDir/buildUpdateSiteDigest.sh -vm $vm -basebuilderDir $basebuilderDir -targetDir $localUpdatesWebDir -trackstats $trackstats";
if [[ $debug -gt 0 ]]; then 
	echo -e '[compose] ['`date +%H:%M:%S`'] Run buildUpdateSiteDigest.sh to create digest.zip ...';
	command=$command" -debug $debug";
	if [[ $debug -gt 1 ]]; then 
		echo "$command" | perl -pe "s/ -/\n  -/g" ; # pretty printing
	fi
fi
$command;

# first try failed, so copy over missing jars from R build and try again
if [[ $? -eq 1 ]] && [[ $subprojectName == "transaction" ]]; then 
	echo -e "[compose] buildUpdateSiteDigest.sh has ${red}FAILED${norm}.";
	canTryAgain=0;
	for b in query validation; do
		siteBackup=$localUpdatesWebDir/${siteLabel}_r0; 
		siteBackup=${siteBackup/-M_r0/-R_r0}; siteBackup=${siteBackup/$buildTypeLabel/releases};
		siteBackup=${siteBackup/-${subprojectName}-/-${b}-}
		if [[ -d $siteBackup ]] && [[ $siteBackup != $localUpdatesWebDir/${siteLabel}_r0 ]]; then # if R folder exists for matching branch, and we're not already doing an R build
			echo -e "          Copy $red$b$norm from $yellow${siteBackup/$localUpdatesWebRoot/}$norm to $blue$localUpdatesWebDir$norm ...";
			cp -r $siteBackup/features/* $localUpdatesWebDir/features/;
			cp -r $siteBackup/plugins/* $localUpdatesWebDir/plugins/;
			canTryAgain=1;
		fi
	done
	if [[ $canTryAgain == 1 ]]; then
		command="$relengCommonScriptsDir/buildUpdateSiteDigest.sh -vm $vm -basebuilderDir $basebuilderDir -targetDir $localUpdatesWebDir -trackstats $trackstats";
		if [[ $debug -gt 0 ]]; then 
			echo -e '[compose] ['`date +%H:%M:%S`'] Run buildUpdateSiteDigest.sh to create digest.zip (2) ...';
			command=$command" -debug $debug";
			if [[ $debug -gt 1 ]]; then 
				echo "$command" | perl -pe "s/ -/\n  -/g" ; # pretty printing
			fi
		fi
		if [[ $debug -eq 0 ]]; then echo "          Trying to generate digest again..."; fi 
		$command;
	fi
fi 

# TODO -- if copy from R fails, try S?
# TODO -- for this to work on remote, we need to add these backup jars to the zip that's pushed to remote

#################################################################################################

# generate local metadata
rm -f $localUpdatesWebDir/artifacts.jar $localUpdatesWebDir/content.jar
command="$relengCommonScriptsDir/buildUpdateSiteMetadata.sh -vm $vm -targetDir $localUpdatesWebDir -trackstats $trackstats -p "$(echo $projectName | tr "a-z" "A-Z");
if [[ $debug -gt 0 ]]; then 
	echo -e '[compose] ['`date +%H:%M:%S`'] Run buildUpdateSiteMetadata.sh to create p2 metadata ...';
	command=$command" -debug $debug";
	if [[ $debug -gt 1 ]]; then 
		echo "$command" | perl -pe "s/ -/\n  -/g" ; # pretty printing
	fi
fi
$command;

#################################################################################################

#promote new stuff to download
if [[ $promote -eq 1 ]]; then
	for i in 0 1; do # same check in logic for sitexml and coordsitexml 
		suf="";
		outfile=$sitexmlOut;
		if [[ $i == 1 ]]; then
			if [[ $coordsiteSuffix ]]; then
				suf=$coordsiteSuffix;
				outfile=$coordsitexmlOut;
			else
				break;
			fi
		fi
		echo "[promote] [`date +%H:%M:%S`] Check in $outfile as /$updatesDirSuffix/$buildTypeLabel/site${suf}.xml ..." ;
		pushd $buildDir/site >/dev/null;
		if [[ ! -d $buildDir/site/$buildTypeLabel ]]; then
			mkdir -p $buildDir/site/$buildTypeLabel;
			cvs -Q add $buildTypeLabel 2>&1 >/dev/null;
		fi
		cp -f $localUpdatesWebDir/$outfile $buildDir/site/$buildTypeLabel/site${suf}.xml;
		pushd $buildDir/site/$buildTypeLabel >/dev/null;
		cvs -Q add site${suf}.xml 2>&1 >/dev/null;
		cvs -d $wwwCVSRep $quietCVS ci -m "buildUpdateSite: $subprojectName $branch $buildID" site${suf}.xml;
		popd >/dev/null;
		popd >/dev/null;
	done
	if [[ $debug -eq 0 ]]; then echo -e "[promote] [`date +%H:%M:%S`] Create $localUpdatesWebRoot/${siteLabel}-${buildID}.zip ..."; fi

	if [[ $debug -gt 0 ]]; then echo -e "[promote] [`date +%H:%M:%S`] Zip ${grey}header.${buildTypeLabel}.site*.xml${norm} ..."; fi
	pushd $buildDir/site >/dev/null;
	zip -9rq $localUpdatesWebRoot/${siteLabel}-${buildID}.zip header.${buildTypeLabel}.site.xml;
	if [[ $coordsiteSuffix ]] && [[ -f header.${buildTypeLabel}.site${coordsiteSuffix}.xml ]]; then 
		zip -9rq $localUpdatesWebRoot/${siteLabel}-${buildID}.zip header.${buildTypeLabel}.site${coordsiteSuffix}.xml;
	fi
	popd >/dev/null;
	pushd $relengCommonScriptsDir >/dev/null;
	if [[ $debug -gt 0 ]]; then echo -e "[promote] [`date +%H:%M:%S`] Zip ${grey}buildUpdateSiteXML.sh${norm}, ${grey}buildUpdateSiteDigest.sh${norm} + ${grey}buildUpdateSiteMetadata.sh${norm}..."; fi
	zip -9rq $localUpdatesWebRoot/${siteLabel}-${buildID}.zip buildUpdateSiteXML.sh buildUpdateSiteDigest.sh buildUpdateSiteMetadata.sh; 
	popd >/dev/null;

	if [[ $debug -gt 0 ]]; then echo -e "[promote] [`date +%H:%M:%S`] Zip $grey${localUpdatesWebDir}${norm}/${blue}jarlist.clean.*.txt$norm ..."; fi
	pushd $localUpdatesWebDir >/dev/null;
	zip -9rq $localUpdatesWebRoot/${siteLabel}-${buildID}.zip jarlist.clean.*.txt; 
	rm -f jarlist.clean.*.txt;
	popd >/dev/null;
	if [[ $debug -gt 0 ]]; then echo -e "[promote] [`date +%H:%M:%S`] Zip $grey${localUpdatesWebDir}/${siteLabel}_r0${norm}/${blue}features${norm} + ${blue}plugins${norm} ..."; fi
	pushd $localUpdatesWebDir/${siteLabel}_r0 >/dev/null;
	zip -9rq $localUpdatesWebRoot/${siteLabel}-${buildID}.zip features plugins;
	popd >/dev/null;
	
	# zip _rX folders' category*.xml files
	if [[ $debug -gt 0 ]]; then echo -e "[promote] [`date +%H:%M:%S`] Zip $blue$numc$norm $grey*_r*${norm} folders' category*.xml files ..."; fi
	rtemp=`mktemp -d`
	for c in $componentSiteDirs; do
		cc=${c##*/}; # m2t-jet-0.8-R_r0
		mkdir $rtemp/$cc;
		if [[ $debug -gt 1 ]]; then	echo "          $cc: "$(cd $c; find . -name "category*.xml"); fi 
		cp -f $c/category*.xml $rtemp/$cc;
	done 
	pushd $rtemp >/dev/null;
	zip -9rq $localUpdatesWebRoot/${siteLabel}-${buildID}.zip *;
	popd >/dev/null;
	rm -fr $rtemp;

	echo -e -n "[promote] Zip contains ${yellow}"
	echo -n $(unzip -l $localUpdatesWebRoot/${siteLabel}-${buildID}.zip | tail -1 | perl -pe 's#.+[^0-9]+([0-9]+) files#\1#')
	echo -e -n " files${norm} and is ${yellow}"
	echo -n $(du -sh $localUpdatesWebRoot/${siteLabel}-${buildID}.zip | perl -pe 's#([0-9]+[mgkbMGKB]+).+#\1#')
	echo -e "${norm}."
	echo -e "[promote] [`date +%H:%M:%S`] Promote zip to $downloadServerFullName..." ;
	scp $quietSSH $localUpdatesWebRoot/${siteLabel}-${buildID}.zip $eclipseSSHUser:~/

	# note: different servers implement sort differently, so diffing the jarlist.*.txt files (for debugging) may not yield
	# expected results. Instead, compare `wc jarlist.clean.*.txt` with `wc jarlist.cleaned.txt`
	if [[ $buildType != "R" ]]; then
		echo -e "[promote] [`date +%H:%M:%S`] Replace content in REMOTE update site ($blue${updatesDir}${norm}) ...";
	else # copy /updates/releases/ into /updates/
		echo -e "[promote] [`date +%H:%M:%S`] Replace content in REMOTE update site & root ($blue${updatesRoot}${norm}/$blue${buildTypeLabel}${norm}) ...";
	fi	
	command="";	
	ssh $eclipseSSHUser "
		echo -e '[promote] Connected to ${yellow}'\$(hostname -f)'${norm}. Unpack '~/${siteLabel}-${buildID}.zip' ... ';
		mkdir -p $updatesDir; 
		pushd $updatesDir >/dev/null; 
		unzip -uo -qq ~/${siteLabel}-${buildID}.zip;
		find features/ plugins/ -type f > $updatesDir/jarlist.dirty.txt;
		if [[ $debug -gt 0 ]]; then  
			echo '[promote] Remove old jars:';
		else
			echo -n '[promote] Remove old jars ';
		fi
		cat jarlist.clean.*.txt > jarlist.clean.txt;
		for f in \$(cat jarlist.dirty.txt); do 
			if [[ \$(grep -c \$f jarlist.clean.txt) -eq 0 ]]; then
				if [[ $debug -gt 0 ]]; then
					echo '          '\$f;
				else
					echo -n '.';
				fi
				rm -f $updatesDir/\$f;
			fi; 
		done
		if [[ $debug -eq 0 ]]; then  
			echo '';
		fi
		chmod +x buildUpdateSiteXML.sh;
		command='./buildUpdateSiteXML.sh -label '$buildTypeLabel' -header header.'$buildTypeLabel' -targetDir '$updatesDir' '\$(find $updatesDir -maxdepth 1 -type d -name \"*_r?\");
		if [[ \"$coordsiteSuffix\" != \"\" ]]; then
			command=\$command' -coordsite '${coordsiteSuffix:1};
		fi
		if [[ $debug -gt 0 ]]; then 
			echo -e '[compose] ['`date +%H:%M:%S`'] Run buildUpdateSiteXML.sh to create site*.xml ...' 
			command=\$command' -debug '$debug;
			if [[ $debug -gt 1 ]]; then 
				echo \"\$command\" | perl -pe 's/ -/\n  -/g' ; # pretty printing
			fi
		fi
		\$command;
		if [[ $debug -gt 1 ]]; then echo '[promote] Remove buildUpdateSiteXML.sh + header.$buildTypeLabel.site*.xml ... '; fi
		rm -f buildUpdateSiteXML.sh header.$buildTypeLabel.site*.xml;
		
		chmod +x buildUpdateSiteDigest.sh;
		rm -f $updatesDir/digest.zip
		command='./buildUpdateSiteDigest.sh -basebuilderDir /opt/public/modeling/build/org.eclipse.releng.basebuilder -targetDir '$updatesDir' -trackstats '$trackstats;
		if [[ $debug -gt 0 ]]; then 
			echo -e '[compose] ['`date +%H:%M:%S`'] Run buildUpdateSiteDigest.sh to create digest.zip ...';
			command=\$command' -debug '$debug;
			if [[ $debug -gt 1 ]]; then 
				echo \"\$command\" | perl -pe 's/ -/\n  -/g' ; # pretty printing
			fi
		fi
		\$command;
		if [[ $debug -gt 1 ]]; then echo '[promote] Remove buildUpdateSiteDigest.sh ... '; fi
		rm -f buildUpdateSiteDigest.sh;

		chmod +x buildUpdateSiteMetadata.sh;
		rm -f $updatesDir/artifacts.jar $updatesDir/content.jar
		command='./buildUpdateSiteMetadata.sh -basebuilderDir /opt/public/modeling/build/org.eclipse.releng.basebuilder -targetDir '$updatesDir' -trackstats '$trackstats' -p '$(echo $projectName | tr "a-z" "A-Z");
		if [[ $debug -gt 0 ]]; then 
			echo -e '[compose] ['`date +%H:%M:%S`'] Run buildUpdateSiteMetadata.sh to create p2 metadata ...';
			command=\$command' -debug '$debug;
			if [[ $debug -gt 1 ]]; then 
				echo \"\$command\" | perl -pe 's/ -/\n  -/g' ; # pretty printing
			fi
		fi
		\$command;
		if [[ $debug -gt 1 ]]; then echo '[promote] Remove buildUpdateSiteMetadata.sh ... '; fi
		rm -f buildUpdateSiteMetadata.sh;

		echo -n '[promote] Remove temp zip & jarlist*.txt ... '; # but keep the server-specific ones!
		rm -f ~/${siteLabel}-${buildID}.zip $updatesDir/jarlist.dirty.txt $updatesDir/jarlist.clean.txt;
		if [[ \"$buildType\" == \"R\" ]]; then
			echo -n 'replace root site... ';
			rm -fr $updatesRoot/artifacts.jar $updatesRoot/content.jar $updatesRoot/digest.zip $updatesRoot/features $updatesRoot/plugins;
			cp -r   $updatesDir/artifacts.jar  $updatesDir/content.jar  $updatesDir/digest.zip  $updatesDir/features  $updatesDir/plugins $updatesDir/site*.xml $updatesRoot;
		fi
		echo 'done.';
		
		echo -n '[promote] Fix perms ... ';
		if [[ \"$buildType\" == \"R\" ]]; then
			chgrp -f -R $eclipseUserGroup $updatesRoot;
			chmod -f -R g+w $updatesRoot;
		else
			chgrp -f -R $eclipseUserGroup $updatesDir;
			chmod -f -R g+w $updatesDir;
		fi
		echo 'done.';
		
		popd >/dev/null;
	";

	# publish new Update zip & md5
	if [[ $(find $buildDropsDir/$branch/$buildID -maxdepth 1 -type f -name "*-Update-*.zip*") ]]; then
		echo -n "[promote] [`date +%H:%M:%S`] SCP Update zip & md5 to remote box... "
		rsync -ar $quietSSH --rsh=ssh $buildDropsDir/$branch/$buildID/*-Update-*.zip* $eclipseSSHUser:$projectDropsDir/$branch/$buildID/
		echo "done [`date +%H:%M:%S`]."
	fi
	
else
	echo "[promote] Check in site*.xml to CVS ... omitted."
	echo "[promote] Promote update site to $downloadServerFullName ... omitted."
fi

#################################################################################################

if [[ $debug -gt 0 ]]; then echo "[ perms ] Fix permissions in $localUpdatesWebDir ..." ; fi
chgrp -f -R $buildUserGroup $localUpdatesWebDir;
chmod -f -R g+w $localUpdatesWebDir;

if [[ $buildType == "R" ]]; then # copy /updates/releases/ into /updates/
	echo -e "[compose] [`date +%H:%M:%S`] Replace content in LOCAL update site root ($blue${localUpdatesWebRoot}${norm}) ...";
	rm -fr $localUpdatesWebRoot/artifacts.jar $localUpdatesWebRoot/content.jar $localUpdatesWebRoot/digest.zip $localUpdatesWebRoot/features $localUpdatesWebRoot/plugins $localUpdatesWebRoot/site*.xml;
	cp -r   $localUpdatesWebDir/artifacts.jar  $localUpdatesWebDir/content.jar  $localUpdatesWebDir/digest.zip  $localUpdatesWebDir/features  $localUpdatesWebDir/plugins  $localUpdatesWebDir/site*.xml $localUpdatesWebRoot;  
	echo -e "[compose] [`date +%H:%M:%S`] Local update site is: $blue$localUpdatesWebRoot$norm/$blue$buildTypeLabel$norm";

	if [[ $debug -gt 0 ]]; then echo "[ perms ] Fix permissions in $localUpdatesWebRoot ..." ; fi
	for d in $localUpdatesWebRoot/digest.zip "$localUpdatesWebRoot/site*.xml" "$localUpdatesWebRoot/site-ganymede.xml" $localUpdatesWebRoot/features $localUpdatesWebRoot/plugins; do
		chgrp -f -R $buildUserGroup $d
		chmod -f -R g+w $d
	done
else 
	echo -e "[compose] [`date +%H:%M:%S`] Local update site is: $blue$localUpdatesWebDir$norm";
fi

#################################################################################################

cleanupAndExit;

########### DONE ###########
